package uoc.multimediadegree.com.e_lactancia_app.modelo;

import android.app.ProgressDialog;
import android.content.ContentValues;
import android.content.Context;
import android.util.Log;
import android.widget.ArrayAdapter;

import com.android.volley.Request;
import com.android.volley.Response;
import com.android.volley.VolleyError;
import com.android.volley.toolbox.JsonObjectRequest;


import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.lang.Override;import java.lang.String;import java.util.ArrayList;
import java.util.List;


public class JSONAdapter extends ArrayAdapter {

    public static final String URL_BASE = "http://e-lactancia.org";
    public static final String URL_JSON_LISTOFTERMS = "/API/get_list_of_terms";
    public static final String URL_JSON_TERM = "/API/get_term/";
    public static final String JSON_SECTION_PRODUCT = "producto";
    public static final String JSON_SECTION_ALTERNATIVES = "alternativas";
    public static final String JSON_SECTION_GROUPS = "grupo";
    public static final String JSON_SECTION_MARKS = "marcas";
    public static final String JSON_SECTION_BIBLIO = "biblio";
    public static final String JSON_SECTION_LISTOFTERMS = "list";
    private static final String TAG = "JSONAdapter";
    private List<JSONFiles.ListOfTerms> items;
    private ProgressDialog loading;
    JsonObjectRequest jsArrayRequest;

    public JSONAdapter(Context context) {
        super(context, 0);
    }

    public void listOfTerms(){
        jsArrayRequest = new JsonObjectRequest(
                Request.Method.GET,
                URL_BASE + URL_JSON_LISTOFTERMS,
                null,
                new Response.Listener<JSONObject>() {
                    @Override
                    public void onResponse(JSONObject response) {
                        items = parseJsonListOfTerms(response);
                        notifyDataSetChanged();
                        fillSearchTable();
                        loading.dismiss();
                    }
                },
                new Response.ErrorListener() {
                    @Override
                    public void onErrorResponse(VolleyError error) {
                        Log.d(TAG, "Error Respuesta en JSON: " + error.getMessage());
                    }
                }
        );
        loading = ProgressDialog.show(getContext(), "Por favor espere...", "Cargando informaicón de inicio... puede durar unos minutos", false, false);
        MySingleton.getInstance(getContext()).addToRequestQueue(jsArrayRequest);
     }

    private void fillSearchTable() {
        getContext().getContentResolver().delete(MyContracts.CONTENT_URI_SEARCH, "1", null);
        getContext().getContentResolver().delete(MyContracts.CONTENT_URI_SUGGEST, "1", null);

        ContentValues values = new ContentValues();
        for (int i = 0; i < items.size(); i++) {
            values.put(MyContracts.Columnas.SEARCH_TERM, items.get(i).getListOfTermsNombre_es());
            values.put(MyContracts.Columnas.SEARCH_REFWEB, items.get(i).getListOfTermsId());
            getContext().getContentResolver().insert(MyContracts.CONTENT_URI_SEARCH, values);
            values.clear();
            values.put(MyContracts.Columnas.SUGGEST_TERM, items.get(i).getListOfTermsNombre_es());
            values.put(MyContracts.Columnas.SUGGEST_OTHERTERM, items.get(i).getListOfTermsNombre_en());
            values.put(MyContracts.Columnas.SUGGEST_REFWEB, items.get(i).getListOfTermsId());
            getContext().getContentResolver().insert(MyContracts.CONTENT_URI_SUGGEST, values);
            values.clear();
        }
    }

    @Override
    public int getCount() {
        return items != null ? items.size() : 0;
    }

    public List<JSONFiles.ListOfTerms> parseJsonListOfTerms(JSONObject jsonObject){
        // Variables locales
        List<JSONFiles.ListOfTerms> listOfTerms_List = new ArrayList<>();
        JSONArray jsonArray= null;
        try {
            // Obtener el array del objeto
            jsonArray = jsonObject.getJSONArray(JSON_SECTION_LISTOFTERMS);
            for(int i=0; i<jsonArray.length(); i++){
                try {
                    JSONObject objeto= jsonArray.getJSONObject(i);

                    JSONFiles.ListOfTerms listOfTerms;
                    listOfTerms = new JSONFiles.ListOfTerms(
                            objeto.getString("id"),
                            objeto.getString("nombre_es"),
                            objeto.getString("nombre_en"));

                    listOfTerms_List.add(listOfTerms);

                } catch (JSONException e) {
                    Log.e(TAG, "Error de parsing: "+ e.getMessage());
                }
            }
        } catch (JSONException e) {
            e.printStackTrace();
        }
        return listOfTerms_List;
    }
}
