package uoc.multimediadegree.com.e_lactancia_app.modelo;

import android.app.SearchManager;
import android.content.ContentProvider;
import android.content.ContentUris;
import android.content.ContentValues;
import android.database.Cursor;
import android.database.SQLException;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteQueryBuilder;
import android.net.Uri;
import android.provider.BaseColumns;
import android.text.TextUtils;

public class MyContentProvider extends ContentProvider {

    private static final String DATABASE_NAME = "elactanciaapp.db";
    private static final int DATABASE_VERSION = 3;

    public DatabaseHelper databaseHelper;

    @Override
    public boolean onCreate() {
        // Inicializando gestor BD
        databaseHelper = new DatabaseHelper(
                getContext(),
                DATABASE_NAME,
                null,
                DATABASE_VERSION
        );
        return true;
    }

    @Override
    public String getType(Uri uri) {
        switch (MyContracts.uriMatcher.match(uri)) {
            case MyContracts.ALLROWS_USUARIO:
                return MyContracts.MULTIPLE_MIME_USUARIO;
            case MyContracts.SINGLE_ROW_USUARIO:
                return MyContracts.SINGLE_MIME_USUARIO;
            case MyContracts.ALLROWS_FAVORITO:
                return MyContracts.MULTIPLE_MIME_FAVORITO;
            case MyContracts.SINGLE_ROW_FAVORITO:
                return MyContracts.SINGLE_MIME_FAVORITO;
            case MyContracts.ALLROWS_HISTORIAL:
                return MyContracts.MULTIPLE_MIME_HISTORIAL;
            case MyContracts.SINGLE_ROW_HISTORIAL:
                return MyContracts.SINGLE_MIME_HISTORIAL;
            case MyContracts.ALLROWS_SEARCH:
                return MyContracts.MULTIPLE_MIME_SEARCH;
            case MyContracts.SINGLE_ROW_SEARCH:
                return MyContracts.SINGLE_MIME_SEARCH;
            case MyContracts.SEARCH_SUGGEST:
                return SearchManager.SUGGEST_MIME_TYPE;
            default:
                throw new IllegalArgumentException("Tipo de actividad desconocida: " + uri);
        }
    }

    private int checkUriMatcher(Uri uri) {
        switch (MyContracts.uriMatcher.match(uri)) {
            case MyContracts.ALLROWS_USUARIO:
                return MyContracts.ALLROWS_USUARIO;
            case MyContracts.ALLROWS_FAVORITO:
                return MyContracts.ALLROWS_FAVORITO;
            case MyContracts.ALLROWS_HISTORIAL:
                return MyContracts.ALLROWS_HISTORIAL;
            case MyContracts.ALLROWS_SEARCH:
                return MyContracts.ALLROWS_SEARCH;
            case MyContracts.SEARCH_SUGGEST:
                return MyContracts.SEARCH_SUGGEST;
            default:
                return -1;
        }
    }


    @Override
    public Cursor query(Uri uri, String[] projection, String selection, String[] selectionArgs, String sortOrder) {
        // Obtener base de datos
        SQLiteDatabase db = databaseHelper.getWritableDatabase();
        // Comparar Uri
        int match = MyContracts.uriMatcher.match(uri);

        Cursor c;
        long videoID;

        switch (match) {
            case MyContracts.ALLROWS_USUARIO:
                // Consultando todos los registros
                c = db.query(MyContracts.USUARIO, projection,
                        selection, selectionArgs,
                        null, null, sortOrder);
                c.setNotificationUri(
                        getContext().getContentResolver(),
                        MyContracts.CONTENT_URI_USUARIO);
                break;
            case MyContracts.SINGLE_ROW_USUARIO:
                // Consultando un solo registro basado en el Id del Uri
                videoID = ContentUris.parseId(uri);
                c = db.query(MyContracts.USUARIO, projection,
                        MyContracts.Columnas._ID + " = " + videoID,
                        selectionArgs, null, null, sortOrder);
                c.setNotificationUri(
                        getContext().getContentResolver(),
                        MyContracts.CONTENT_URI_USUARIO);
                break;
            case MyContracts.ALLROWS_FAVORITO:
                // Consultando todos los registros
                c = db.query(MyContracts.FAVORITO, projection,
                        selection, selectionArgs,
                        null, null, sortOrder);
                c.setNotificationUri(
                        getContext().getContentResolver(),
                        MyContracts.CONTENT_URI_FAVORITO);
                break;
            case MyContracts.SINGLE_ROW_FAVORITO:
                // Consultando un solo registro basado en el Id del Uri
                videoID = ContentUris.parseId(uri);
                c = db.query(MyContracts.FAVORITO, projection,
                        MyContracts.Columnas.FAV_REFWEB + " = " + videoID,
                        selectionArgs, null, null, sortOrder);
                c.setNotificationUri(
                        getContext().getContentResolver(),
                        MyContracts.CONTENT_URI_FAVORITO);
                break;
            case MyContracts.ALLROWS_HISTORIAL:
                // Consultando todos los registros
                c = db.query(MyContracts.HISTORIAL, projection,
                        selection, selectionArgs,
                        null, null, sortOrder);
                c.setNotificationUri(
                        getContext().getContentResolver(),
                        MyContracts.CONTENT_URI_HISTORIAL);
                break;
            case MyContracts.SINGLE_ROW_HISTORIAL:
                // Consultando un solo registro basado en el Id del Uri
                videoID = ContentUris.parseId(uri);
                c = db.query(MyContracts.HISTORIAL, projection,
                        MyContracts.Columnas.HIST_REFWEB + " = " + videoID,
                        selectionArgs, null, null, sortOrder);
                c.setNotificationUri(
                        getContext().getContentResolver(),
                        MyContracts.CONTENT_URI_HISTORIAL);
                break;
            case MyContracts.ALLROWS_SUGGEST:
                // Consultando todos los registros
                c = db.query(MyContracts.FTS_VIRTUAL_TABLE, projection,
                        selection, selectionArgs,
                        null, null, sortOrder);
                c.setNotificationUri(
                        getContext().getContentResolver(),
                        MyContracts.CONTENT_URI_SUGGEST);
                break;
            case MyContracts.SINGLE_ROW_SUGGEST:
                // Consultando un solo registro basado en el Id del Uri
                videoID = ContentUris.parseId(uri);
                c = db.query(MyContracts.FTS_VIRTUAL_TABLE, projection,
                        MyContracts.Columnas.SUGGEST_REFWEB + " = " + videoID,
                        selectionArgs, null, null, sortOrder);
                c.setNotificationUri(
                        getContext().getContentResolver(),
                        MyContracts.CONTENT_URI_SUGGEST);
                break;
            case MyContracts.SEARCH_SUGGEST:
                if (selectionArgs == null) {
                    try{
                        c = db.query("FTSSuggest_content", new String[]{"c2suggest_intent_data"}, "docid =?",
                                new String[]{selection}, null, null, null, null);
                        return c;
                    } catch (Exception E) {
                        throw new IllegalArgumentException(
                                "selectionArgs must be provided for the Uri: " + uri);
                    }
                }
                return getSuggestions(selectionArgs[0]);
            case MyContracts.ALLROWS_SEARCH:
                if (selectionArgs == null) {
                    throw new IllegalArgumentException(
                            "selectionArgs must be provided for the Uri: " + uri);
                }
                return search(selectionArgs[0]);
            case MyContracts.SINGLE_ROW_SEARCH:
                return getWord(uri);
            default:
                throw new IllegalArgumentException("URI no soportada: " + uri);
        }
        return c;
    }

    @Override
    public Uri insert(Uri uri, ContentValues values) {
        // Validar la uri
        int uriMatch = checkUriMatcher(uri);
        if (uriMatch == -1) {
            throw new IllegalArgumentException("URI desconocida : " + uri);
        }
        ContentValues contentValues;
        if (values != null) {
            contentValues = new ContentValues(values);
        } else {
            contentValues = new ContentValues();
        }

        // Inserción de nueva fila
        SQLiteDatabase db = databaseHelper.getWritableDatabase();
        long rowId;
        switch (uriMatch) {
            case MyContracts.ALLROWS_USUARIO:
                rowId = db.insert(MyContracts.USUARIO,
                        null, contentValues);
                if (rowId > 0) {
                    Uri uri_actividad =
                            ContentUris.withAppendedId(
                                    MyContracts.CONTENT_URI_USUARIO, rowId);
                    getContext().getContentResolver().
                            notifyChange(uri_actividad, null);
                    return uri_actividad;
                }
                break;
            case MyContracts.ALLROWS_FAVORITO:
                rowId = db.insert(MyContracts.FAVORITO,
                        null, contentValues);
                if (rowId > 0) {
                    Uri uri_actividad =
                            ContentUris.withAppendedId(
                                    MyContracts.CONTENT_URI_FAVORITO, rowId);
                    getContext().getContentResolver().
                            notifyChange(uri_actividad, null);
                    return uri_actividad;
                }
                break;
            case MyContracts.ALLROWS_HISTORIAL:
                rowId = db.insert(MyContracts.HISTORIAL,
                        null, contentValues);
                if (rowId > 0) {
                    Uri uri_actividad =
                            ContentUris.withAppendedId(
                                    MyContracts.CONTENT_URI_HISTORIAL, rowId);
                    getContext().getContentResolver().
                            notifyChange(uri_actividad, null);
                    return uri_actividad;
                }
                break;
            case MyContracts.ALLROWS_SEARCH:
                rowId = db.insert(MyContracts.SEARCH,
                        null, contentValues);
                if (rowId > 0) {
                    Uri uri_actividad =
                            ContentUris.withAppendedId(
                                    MyContracts.CONTENT_URI_SEARCH, rowId);
                    getContext().getContentResolver().
                            notifyChange(uri_actividad, null);
                    return uri_actividad;
                }
                break;
            case MyContracts.SEARCH_SUGGEST:
                rowId = db.insert(MyContracts.FTS_VIRTUAL_TABLE,
                        null, contentValues);
                if (rowId > 0) {
                    Uri uri_actividad =
                            ContentUris.withAppendedId(
                                    MyContracts.CONTENT_URI_SUGGEST, rowId);
                    getContext().getContentResolver().
                            notifyChange(uri_actividad, null);
                    return uri_actividad;
                }
                break;
        }
        throw new SQLException("Falla al insertar fila en : " + uri);
    }

    @Override
    public int delete(Uri uri, String selection, String[] selectionArgs) {

        SQLiteDatabase db = databaseHelper.getWritableDatabase();

        int match = MyContracts.uriMatcher.match(uri);
        int affected;
        long videoId;

        switch (match) {
            case MyContracts.ALLROWS_USUARIO:
                affected = db.delete(MyContracts.USUARIO,
                        selection,
                        selectionArgs);
                break;
            case MyContracts.SINGLE_ROW_USUARIO:
                videoId = ContentUris.parseId(uri);
                affected = db.delete(MyContracts.USUARIO,
                        MyContracts.Columnas._ID + "=" + videoId
                                + (!TextUtils.isEmpty(selection) ?
                                " AND (" + selection + ')' : ""),
                        selectionArgs);
                // Notificar cambio asociado a la uri
                getContext().getContentResolver().
                        notifyChange(uri, null);
                break;
            case MyContracts.ALLROWS_FAVORITO:
                affected = db.delete(MyContracts.FAVORITO,
                        selection,
                        selectionArgs);
                break;
            case MyContracts.SINGLE_ROW_FAVORITO:
                videoId = ContentUris.parseId(uri);
                affected = db.delete(MyContracts.FAVORITO,
                        MyContracts.Columnas._ID + "=" + videoId
                                + (!TextUtils.isEmpty(selection) ?
                                " AND (" + selection + ')' : ""),
                        selectionArgs);
                // Notificar cambio asociado a la uri
                getContext().getContentResolver().
                        notifyChange(uri, null);
                break;
            case MyContracts.ALLROWS_HISTORIAL:
                affected = db.delete(MyContracts.HISTORIAL,
                        selection,
                        selectionArgs);
                break;
            case MyContracts.SINGLE_ROW_HISTORIAL:
                videoId = ContentUris.parseId(uri);
                affected = db.delete(MyContracts.HISTORIAL,
                        MyContracts.Columnas._ID + "=" + videoId
                                + (!TextUtils.isEmpty(selection) ?
                                " AND (" + selection + ')' : ""),
                        selectionArgs);
                // Notificar cambio asociado a la uri
                getContext().getContentResolver().
                        notifyChange(uri, null);
                break;
            case MyContracts.ALLROWS_SEARCH:
                affected = db.delete(MyContracts.SEARCH,
                        selection,
                        selectionArgs);
                break;
            case MyContracts.SINGLE_ROW_SEARCH:
                videoId = ContentUris.parseId(uri);
                affected = db.delete(MyContracts.SEARCH,
                        MyContracts.Columnas._ID + "=" + videoId
                                + (!TextUtils.isEmpty(selection) ?
                                " AND (" + selection + ')' : ""),
                        selectionArgs);
                // Notificar cambio asociado a la uri
                getContext().getContentResolver().
                        notifyChange(uri, null);
                break;
            case MyContracts.SEARCH_SUGGEST:
                affected = db.delete(MyContracts.FTS_VIRTUAL_TABLE,
                        selection,
                        selectionArgs);
                break;
            default:
                throw new IllegalArgumentException("Elemento actividad desconocido: " +
                        uri);
        }
        return affected;
    }

    @Override
    public int update(Uri uri, ContentValues values, String selection, String[] selectionArgs) {

        SQLiteDatabase db = databaseHelper.getWritableDatabase();
        int affected;
        String videoId;
        switch (MyContracts.uriMatcher.match(uri)) {
            case MyContracts.ALLROWS_USUARIO:
                affected = db.update(MyContracts.USUARIO, values,
                        selection, selectionArgs);
                break;
            case MyContracts.SINGLE_ROW_USUARIO:
                videoId = uri.getPathSegments().get(1);
                affected = db.update(MyContracts.USUARIO, values,
                        MyContracts.Columnas._ID + "=" + videoId
                                + (!TextUtils.isEmpty(selection) ?
                                " AND (" + selection + ')' : ""),
                        selectionArgs);
                break;
            case MyContracts.ALLROWS_FAVORITO:
                affected = db.update(MyContracts.FAVORITO, values,
                        selection, selectionArgs);
                break;
            case MyContracts.SINGLE_ROW_FAVORITO:
                videoId = uri.getPathSegments().get(1);
                affected = db.update(MyContracts.FAVORITO, values,
                        MyContracts.Columnas._ID + "=" + videoId
                                + (!TextUtils.isEmpty(selection) ?
                                " AND (" + selection + ')' : ""),
                        selectionArgs);
                break;
            case MyContracts.ALLROWS_HISTORIAL:
                affected = db.update(MyContracts.HISTORIAL, values,
                        selection, selectionArgs);
                break;
            case MyContracts.SINGLE_ROW_HISTORIAL:
                videoId = uri.getPathSegments().get(1);
                affected = db.update(MyContracts.HISTORIAL, values,
                        MyContracts.Columnas._ID + "=" + videoId
                                + (!TextUtils.isEmpty(selection) ?
                                " AND (" + selection + ')' : ""),
                        selectionArgs);
                break;
            case MyContracts.ALLROWS_SEARCH:
                affected = db.update(MyContracts.SEARCH, values,
                        selection, selectionArgs);
                break;
            case MyContracts.SINGLE_ROW_SEARCH:
                videoId = uri.getPathSegments().get(1);
                affected = db.update(MyContracts.SEARCH, values,
                        MyContracts.Columnas._ID + "=" + videoId
                                + (!TextUtils.isEmpty(selection) ?
                                " AND (" + selection + ')' : ""),
                        selectionArgs);
                break;
            default:
                throw new IllegalArgumentException("URI desconocida: " + uri);
        }
        getContext().getContentResolver().notifyChange(uri, null);
        return affected;
    }

    private Cursor getSuggestions(String query) {
        query = query.toLowerCase();
        String[] columns = new String[]{
                BaseColumns._ID,
                MyContracts.Columnas.SUGGEST_TERM,
                MyContracts.Columnas.SUGGEST_OTHERTERM,
       /* SearchManager.SUGGEST_COLUMN_SHORTCUT_ID,
                        (only if you want to refresh shortcuts) */
                SearchManager.SUGGEST_COLUMN_INTENT_DATA_ID};

        return getWordMatches(query, columns);
    }

    private Cursor search(String query) {
        query = query.toLowerCase();
        String[] columns = new String[]{
                BaseColumns._ID,
                MyContracts.Columnas.SUGGEST_TERM,
                MyContracts.Columnas.SUGGEST_OTHERTERM,
                SearchManager.SUGGEST_COLUMN_INTENT_DATA_ID};

        return getWordMatches(query, columns);
    }

    private Cursor getWord(Uri uri) {
        String rowId = uri.getLastPathSegment();
        String[] columns = new String[]{
                MyContracts.Columnas.SUGGEST_TERM,
                MyContracts.Columnas.SUGGEST_OTHERTERM,
                SearchManager.SUGGEST_COLUMN_INTENT_DATA_ID};

        return getWord(rowId, columns);
    }

    /**
     * Returns a Cursor positioned at the word specified by rowId
     *
     * @param rowId   id of word to retrieve
     * @param columns The columns to include, if null then all are included
     * @return Cursor positioned to matching word, or null if not found.
     */
    public Cursor getWord(String rowId, String[] columns) {
        String selection = "rowid = ?";
        String[] selectionArgs = new String[]{rowId};

        return query(selection, selectionArgs, columns);
   }

    /**
     * Returns a Cursor over all words that match the given query
     *
     * @param query   The string to search for
     * @param columns The columns to include, if null then all are included
     * @return Cursor over all words that match, or null if none found.
     */
    public Cursor getWordMatches(String query, String[] columns) {
        String selection = MyContracts.Columnas.SUGGEST_TERM + " MATCH ?";
        String[] selectionArgs = new String[]{query + "*"};

        return query(selection, selectionArgs, columns);

    }

    /**
     * Performs a database query.
     *
     * @param selection     The selection clause
     * @param selectionArgs Selection arguments for "?" components in the selection
     * @param columns       The columns to return
     * @return A Cursor over all rows matching the query
     */
    public Cursor query(String selection, String[] selectionArgs, String[] columns) {
        /* The SQLiteBuilder provides a map for all possible columns requested to
         * actual columns in the database, creating a simple column alias mechanism
         * by which the ContentProvider does not need to know the real column names
         */
        SQLiteQueryBuilder builder = new SQLiteQueryBuilder();
        builder.setTables(MyContracts.FTS_VIRTUAL_TABLE);
        builder.setProjectionMap(MyContracts.mColumnMap);

        Cursor cursor = builder.query(databaseHelper.getReadableDatabase(),
                columns, selection, selectionArgs, null, null, null);

        if (cursor == null) {
            return null;
        } else if (!cursor.moveToFirst()) {
            cursor.close();
            return null;
        }
        return cursor;
    }
}