package uoc.multimediadegree.com.e_lactancia_app.modelo;

import android.app.SearchManager;
import android.content.UriMatcher;
import android.net.Uri;
import android.provider.BaseColumns;

import java.util.HashMap;

/**
 * Contract Class entre el provider y las aplicaciones
 */
public class MyContracts {
    /**
     * Autoridad del Content Provider
     */
    public final static String AUTHORITY = "uoc.multimediadegree.com.e_lactancia_app.modelo.MyContentProvider";
    /**
     * Representación de la tabla a consultar
     */
    public static final String USUARIO = "usuario";
    public static final String FAVORITO = "favorito";
    public static final String HISTORIAL = "historial";
    public static final String SEARCH = "search";
    public static final String FTS_VIRTUAL_TABLE = "FTSSuggest";

    /**
     * Tipo MIME que retorna la consulta de una sola fila
     */
    public final static String SINGLE_MIME_USUARIO =
            "vnd.android.cursor.item/vnd." + AUTHORITY + USUARIO;
    public final static String SINGLE_MIME_FAVORITO =
            "vnd.android.cursor.item/vnd." + AUTHORITY + FAVORITO;
    public final static String SINGLE_MIME_HISTORIAL =
            "vnd.android.cursor.item/vnd." + AUTHORITY + HISTORIAL;
    public final static String SINGLE_MIME_SEARCH =
            "vnd.android.cursor.item/vnd." + AUTHORITY + SEARCH;
    public final static String SINGLE_MIME_SUGGEST =
            "vnd.android.cursor.item/vnd." + AUTHORITY + FTS_VIRTUAL_TABLE;
    /**
     * Tipo MIME que retorna la consulta de {@link @CONTENT_URI_*}
     */
    public final static String MULTIPLE_MIME_USUARIO =
            "vnd.android.cursor.dir/vnd." + AUTHORITY + USUARIO;
    public final static String MULTIPLE_MIME_FAVORITO =
            "vnd.android.cursor.dir/vnd." + AUTHORITY + FAVORITO;
    public final static String MULTIPLE_MIME_HISTORIAL =
            "vnd.android.cursor.dir/vnd." + AUTHORITY + HISTORIAL;
    public final static String MULTIPLE_MIME_SEARCH =
            "vnd.android.cursor.dir/vnd." + AUTHORITY + SEARCH;
    public final static String MULTIPLE_MIME_SUGGEST =
            "vnd.android.cursor.dir/vnd." + AUTHORITY + FTS_VIRTUAL_TABLE;

    /**
     * URI de contenido principal
     */
    public final static Uri CONTENT_URI_USUARIO =
            Uri.parse("content://" + AUTHORITY + "/" + USUARIO);
    public final static Uri CONTENT_URI_FAVORITO =
            Uri.parse("content://" + AUTHORITY + "/" + FAVORITO);
    public final static Uri CONTENT_URI_HISTORIAL =
            Uri.parse("content://" + AUTHORITY + "/" + HISTORIAL);
    public final static Uri CONTENT_URI_SEARCH =
            Uri.parse("content://" + AUTHORITY + "/" + SEARCH);
    public final static Uri CONTENT_URI_SUGGEST =
            Uri.parse("content://" + AUTHORITY + "/" + FTS_VIRTUAL_TABLE);
    /**
     * Comparador de URIs de contenido
     */
    public static final UriMatcher uriMatcher;
    /**
     * Código para URIs de multiples registros
     */
    public static final int ALLROWS_USUARIO = 0;
    public static final int ALLROWS_FAVORITO = 2;
    public static final int ALLROWS_HISTORIAL = 4;
    public static final int ALLROWS_SEARCH = 6;
    public static final int ALLROWS_SUGGEST = 8;
    /**
     * Código para URIS de un solo registro
     */
    public static final int SINGLE_ROW_USUARIO = 1;
    public static final int SINGLE_ROW_FAVORITO = 3;
    public static final int SINGLE_ROW_HISTORIAL = 5;
    public static final int SINGLE_ROW_SEARCH = 7;
    public static final int SINGLE_ROW_SUGGEST = 9;

    public static final int SEARCH_SUGGEST = 10;

    // Asignación de URIs
    static {
        uriMatcher = new UriMatcher(UriMatcher.NO_MATCH);
        uriMatcher.addURI(AUTHORITY, USUARIO, ALLROWS_USUARIO);
        uriMatcher.addURI(AUTHORITY, USUARIO + "/#", SINGLE_ROW_USUARIO);
        uriMatcher.addURI(AUTHORITY, FAVORITO, ALLROWS_FAVORITO);
        uriMatcher.addURI(AUTHORITY, FAVORITO + "/#", SINGLE_ROW_FAVORITO);
        uriMatcher.addURI(AUTHORITY, HISTORIAL, ALLROWS_HISTORIAL);
        uriMatcher.addURI(AUTHORITY, HISTORIAL + "/#", SINGLE_ROW_HISTORIAL);
        uriMatcher.addURI(AUTHORITY, SEARCH, ALLROWS_SEARCH);
        uriMatcher.addURI(AUTHORITY, SEARCH + "/#", SINGLE_ROW_SEARCH);
        uriMatcher.addURI(AUTHORITY, FTS_VIRTUAL_TABLE, ALLROWS_SUGGEST);
        uriMatcher.addURI(AUTHORITY, FTS_VIRTUAL_TABLE + "/#", SINGLE_ROW_SUGGEST);
        uriMatcher.addURI(AUTHORITY, FTS_VIRTUAL_TABLE, SEARCH_SUGGEST);
        uriMatcher.addURI(AUTHORITY, FTS_VIRTUAL_TABLE + "/*", SEARCH_SUGGEST);
        uriMatcher.addURI(AUTHORITY, SearchManager.SUGGEST_URI_PATH_QUERY, SEARCH_SUGGEST);
        uriMatcher.addURI(AUTHORITY, SearchManager.SUGGEST_URI_PATH_QUERY + "/*", SEARCH_SUGGEST);
    }

    /**
     * Estructura de la tabla
     */
    public static class Columnas implements BaseColumns {

        private Columnas() {
            // Sin instancias
        }

        /*** USUARIO ***/
        public final static String USER_NAME = "nombre";
        public final static String USER_EMAIL = "correo";
        public final static String USER_PROFLIE = "perfil";

        /*** FAVORITO ***/
        public final static String FAV_TERM = "termino";
        public final static String FAV_RISK = "riesgo";
        public final static String FAV_REFWEB = "ref_web";

        /*** HISTORIAL ***/
        public final static String HIST_TERM = "termino";
        public final static String HIST_DATE = "fecha";
        public final static String HIST_REFWEB = "ref_web";

        /*** SEARCH ***/
        public final static String SEARCH_TERM = "termino";
        public final static String SEARCH_REFWEB = "ref_web";

        /*** SUGGEST ***/
        public static final String SUGGEST_TERM = SearchManager.SUGGEST_COLUMN_TEXT_1;
        public static final String SUGGEST_OTHERTERM = SearchManager.SUGGEST_COLUMN_TEXT_2;
        public static final String SUGGEST_REFWEB = SearchManager.SUGGEST_COLUMN_INTENT_DATA;
    }

    public static final HashMap<String,String> mColumnMap = buildColumnMap();
    /**
     * Builds a map for all columns that may be requested, which will be given to the
     * SQLiteQueryBuilder. This is a good way to define aliases for column names, but must include
     * all columns, even if the value is the key. This allows the ContentProvider to request
     * columns w/o the need to know real column names and create the alias itself.
     */
    public static HashMap<String,String> buildColumnMap() {
        HashMap<String,String> map = new HashMap<String,String>();
        map.put(Columnas.SUGGEST_TERM, Columnas.SUGGEST_TERM);
        map.put(Columnas.SUGGEST_OTHERTERM, Columnas.SUGGEST_OTHERTERM);
        map.put(BaseColumns._ID, "rowid AS " +
                BaseColumns._ID);
        map.put(SearchManager.SUGGEST_COLUMN_INTENT_DATA_ID, "rowid AS " +
                SearchManager.SUGGEST_COLUMN_INTENT_DATA_ID);
        map.put(SearchManager.SUGGEST_COLUMN_INTENT_DATA,
                SearchManager.SUGGEST_COLUMN_INTENT_DATA);
        return map;
    }

}
