package uoc.multimediadegree.com.e_lactancia_app.ui;

import android.app.SearchManager;
import android.content.ContentValues;
import android.content.Context;
import android.content.Intent;
import android.database.Cursor;
import android.net.Uri;
import android.os.Bundle;
import android.support.design.widget.NavigationView;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentManager;
import android.support.v4.view.GravityCompat;
import android.support.v4.view.MenuItemCompat;
import android.support.v4.widget.DrawerLayout;
import android.support.v7.app.ActionBar;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.support.v7.widget.SearchView;
import android.support.v7.widget.Toolbar;
import android.view.LayoutInflater;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;

import java.text.SimpleDateFormat;
import java.util.Date;

import uoc.multimediadegree.com.e_lactancia_app.R;
import uoc.multimediadegree.com.e_lactancia_app.modelo.JSONAdapter;
import uoc.multimediadegree.com.e_lactancia_app.modelo.MyContracts;

public class ActividadPrincipal extends AppCompatActivity implements LoginDialog.OnLoginDialogListener{

    private DrawerLayout drawerLayout;
    private SearchView searchView;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.actividad_principal);

        agregarToolbar();

        drawerLayout = (DrawerLayout) findViewById(R.id.drawer_layout);
        NavigationView navigationView = (NavigationView) findViewById(R.id.nav_view);

        if (navigationView != null) {
            prepararDrawer(navigationView);
            // Seleccionar item por defecto
            seleccionarItem(navigationView.getMenu().getItem(0));
        }
        handleIntent(getIntent());

        if( !checkUser() ) {
            new LoginDialog().show(getSupportFragmentManager(), "LoginDialog");
        }
        fillSuggestTable();
    }

    private boolean isEmptySuggestTable(){
        Cursor c = getContentResolver().query(MyContracts.CONTENT_URI_SUGGEST, null, "1", null, null, null);
        return !c.moveToFirst();
    }

    private boolean checkUser(){
        Cursor c = getContentResolver().query(MyContracts.CONTENT_URI_USUARIO, null, null, null, null, null);
        return c.moveToFirst();
    }

    @Override
    public void onPossitiveButtonClick() {

    }

    private void fillSuggestTable() {
        if(isEmptySuggestTable()){
            if(checkUser()) {
                JSONAdapter JSONAdapter;
                JSONAdapter = new JSONAdapter(this);
                JSONAdapter.listOfTerms();
            }
        }
    }

    @Override
    protected void onNewIntent(Intent intent) {
        setIntent(intent);
        handleIntent(intent);
    }

    private void handleIntent(Intent intent) {
        if (Intent.ACTION_VIEW.equals(intent.getAction())) {
            searchView.onActionViewCollapsed();
            searchView.setQuery("", false);
            searchView.clearFocus();

            // handles a click on a search suggestion; launches activity to show word
            Uri detailUri = intent.getData();
            String id = detailUri.getLastPathSegment();
            Intent detailIntent = new Intent(this, ActividadDetalleTermino.class);
            detailIntent.putExtra("id", id);
            startActivity(detailIntent);
        }
    }

    private void agregarToolbar() {
        Toolbar toolbar = (Toolbar) findViewById(R.id.toolbar);
        setSupportActionBar(toolbar);
        final ActionBar ab = getSupportActionBar();
        if (ab != null) {
            // Poner ícono del drawer toggle
            ab.setHomeAsUpIndicator(R.drawable.drawer_toggle);
            ab.setDisplayHomeAsUpEnabled(true);
        }
    }

    private void prepararDrawer(NavigationView navigationView) {
        navigationView.setNavigationItemSelectedListener(
                new NavigationView.OnNavigationItemSelectedListener() {
                    @Override
                    public boolean onNavigationItemSelected(MenuItem menuItem) {
                        menuItem.setChecked(true);
                        seleccionarItem(menuItem);
                        drawerLayout.closeDrawers();
                        return true;
                    }
                });
    }


    private void seleccionarItem(MenuItem itemDrawer) {
        Fragment fragmentoGenerico = null;
        FragmentManager fragmentManager = getSupportFragmentManager();

        switch (itemDrawer.getItemId()) {
            case R.id.item_inicio:
                fragmentoGenerico = new FragmentoInicio();
                break;
            case R.id.item_cuenta:
                fragmentoGenerico = new FragmentoCuenta();
                ((FragmentoCuenta) fragmentoGenerico).setPestanaDefecto(0);
                break;
            case R.id.item_favoritos:
                fragmentoGenerico = new FragmentoCuenta();
                ((FragmentoCuenta) fragmentoGenerico).setPestanaDefecto(1);
                break;
            case R.id.item_historial:
                fragmentoGenerico = new FragmentoCuenta();
                ((FragmentoCuenta) fragmentoGenerico).setPestanaDefecto(2);
                break;
            case R.id.item_estudio:
                startActivity(new Intent(this, ActividadEstudio.class));
                break;
        }
        if (fragmentoGenerico != null) {
            fragmentManager
                    .beginTransaction()
                    .replace(R.id.contenedor_principal, fragmentoGenerico)
                    .commit();
        }
        // Setear título actual
        setTitle(itemDrawer.getTitle());
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        super.onCreateOptionsMenu(menu);
        getMenuInflater().inflate(R.menu.menu_principal, menu);

        MenuItem searchItem = menu.findItem(R.id.action_search);
        // Get the SearchView and set the searchable configuration
        SearchManager searchManager = (SearchManager) getSystemService(Context.SEARCH_SERVICE);
        searchView = (SearchView) MenuItemCompat.getActionView(searchItem);
        // Assumes current activity is the searchable activity
        searchView.setSearchableInfo(searchManager.getSearchableInfo(getComponentName()));
        searchView.setIconifiedByDefault(true); // Iconify the widget
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        switch (item.getItemId()) {
            case android.R.id.home:
                drawerLayout.openDrawer(GravityCompat.START);
                return true;
        }
        return super.onOptionsItemSelected(item);
    }
}