package uoc.multimediadegree.com.e_lactancia_app.ui;

import android.app.Activity;
import android.app.Dialog;
import android.content.ContentValues;
import android.content.Context;
import android.content.DialogInterface;
import android.database.Cursor;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.v4.app.DialogFragment;
import android.support.v7.app.AlertDialog;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.EditText;
import android.widget.Spinner;
import android.widget.TextView;
import android.widget.Toast;

import java.text.SimpleDateFormat;
import java.util.Date;

import uoc.multimediadegree.com.e_lactancia_app.R;
import uoc.multimediadegree.com.e_lactancia_app.modelo.JSONAdapter;
import uoc.multimediadegree.com.e_lactancia_app.modelo.MyContracts;

/**
 * Fragmento con diálogo básico
 */
public class LoginDialog extends DialogFragment  {

    private Button signin;
    private EditText userName;
    private EditText userEMail;
    private Spinner userProfile;

    public interface OnLoginDialogListener {
        void onPossitiveButtonClick();// Eventos Botón Positivo
    }

    // Interfaz de comunicación
    OnLoginDialogListener listener;

    public LoginDialog() {
    }

    @NonNull
    @Override
    public Dialog onCreateDialog(Bundle savedInstanceState) {
        return createLoginDialog();
    }

    public AlertDialog createLoginDialog() {
        AlertDialog.Builder builder = new AlertDialog.Builder(getActivity());

        LayoutInflater inflater = getActivity().getLayoutInflater();
        View view = inflater.inflate(R.layout.dialog_signin, null);
        builder.setView(view);

        signin = (Button) view.findViewById(R.id.entrar_boton);
        userName = (EditText) view.findViewById(R.id.login_txt_nombre);
        userEMail = (EditText) view.findViewById(R.id.login_txt_email);
        userProfile = (Spinner) view.findViewById(R.id.login_spin_profile);

        builder.setView(view);

        signin.setOnClickListener(
                new View.OnClickListener() {
                    @Override
                    public void onClick(View v) {
                        if (checkLoginInfo()) {
                            ContentValues values = new ContentValues();

                            values.put(MyContracts.Columnas.USER_NAME, userName.getText().toString());
                            values.put(MyContracts.Columnas.USER_EMAIL, userEMail.getText().toString());
                            values.put(MyContracts.Columnas.USER_PROFLIE, userProfile.getSelectedItem().toString());

                            getActivity().getContentResolver().insert(MyContracts.CONTENT_URI_USUARIO, values);

                            dismiss();
                            fillSuggestTable();
                        } else {
                            Toast.makeText(getActivity(), getResources().getString(R.string.login_dataRequired), Toast.LENGTH_LONG).show();
                        }
                    }
                }
        );
        return builder.create();
    }

    private void fillSuggestTable() {
        if(checkUser()) {
            JSONAdapter JSONAdapter;
            JSONAdapter = new JSONAdapter(getActivity());
            JSONAdapter.listOfTerms();
        }
    }

    private boolean checkLoginInfo(){
        if( userName.getText().toString() != ""){
            if( userEMail.getText().toString() != "") {
                return true;
            }
        }
        return false;
    }

    @Override
    public void onAttach(Activity activity) {
        super.onAttach(activity);

        try {
            listener = (OnLoginDialogListener) activity;
        } catch (ClassCastException e) {
            throw new ClassCastException(
                    activity.toString() +
                            " no implementó OnSimpleDialogListener");

        }
    }

    @Override
    public void onDismiss(DialogInterface dialog) {
        if( !checkUser()){
            Toast.makeText(getActivity(), getResources().getString(R.string.login_loginRequired), Toast.LENGTH_LONG).show();
            getActivity().finish();
        }
    }

    private boolean checkUser(){
        Cursor c = getActivity().getContentResolver().query(MyContracts.CONTENT_URI_USUARIO, null, null, null, null, null);
        return c.moveToFirst();
    }
}